"""
Detector widgets
"""

# Standard library modules.
from operator import methodcaller

# Third party modules.
from PySide.QtGui import \
    QComboBox, QToolBar, QMessageBox, QTableView, QItemDelegate
from PySide.QtCore import Qt, QAbstractTableModel, QModelIndex

# Local modules.
from pyhmsa.gui.util.parameter import \
    (ParameterWidget, NumericalAttributeLineEdit,
     TextAttributeLineEdit, UnitAttributeLineEdit)
from pyhmsa.gui.util.icon import getIcon
from pyhmsa.gui.spec.condition.condition import _ConditionWidget
from pyhmsa.gui.spec.condition.calibration import CalibrationWidget

from pyhmsa.spec.condition.detector import \
    (PulseHeightAnalyser, Window, WindowLayer,
     DetectorCamera, DetectorSpectrometer, DetectorSpectrometerCL,
     DetectorSpectrometerWDS, DetectorSpectrometerXEDS)
from pyhmsa.spec.condition.calibration import CalibrationConstant

# Globals and constants variables.
from pyhmsa.spec.condition.detector import \
    _SIGNAL_TYPES, _COLLECTION_MODES, _PHA_MODES, _XEDS_TECHNOLOGIES

class PulseHeightAnalyserWidget(ParameterWidget):

    def __init__(self, parent=None):
        ParameterWidget.__init__(self, PulseHeightAnalyser, parent)

    def _init_ui(self):
        # Widgets
        self._txt_bias = NumericalAttributeLineEdit(self.CLASS.bias)
        self._txt_gain = NumericalAttributeLineEdit(self.CLASS.gain)
        self._txt_base_level = NumericalAttributeLineEdit(self.CLASS.base_level)
        self._txt_window = NumericalAttributeLineEdit(self.CLASS.window)
        self._cb_mode = QComboBox()
        self._cb_mode.addItems([None] + list(_PHA_MODES))

        # Layouts
        layout = ParameterWidget._init_ui(self)
        layout.addRow('Bias', self._txt_bias)
        layout.addRow('Gain', self._txt_gain)
        layout.addRow('Base level', self._txt_base_level)
        layout.addRow('Window', self._txt_window)
        layout.addRow('Mode', self._cb_mode)

        # Signals
        self._txt_bias.textEdited.connect(self.edited)
        self._txt_gain.textEdited.connect(self.edited)
        self._txt_base_level.textEdited.connect(self.edited)
        self._txt_window.textEdited.connect(self.edited)
        self._cb_mode.currentIndexChanged.connect(self.edited)

        return layout

    def parameter(self, parameter=None):
        parameter = ParameterWidget.parameter(self, parameter)
        parameter.bias = self._txt_bias.text()
        parameter.gain = self._txt_gain.text()
        parameter.base_level = self._txt_base_level.text()
        parameter.window = self._txt_window.text()
        parameter.mode = self._cb_mode.currentText()
        return parameter

    def setParameter(self, parameter):
        ParameterWidget.setParameter(self, parameter)
        self._txt_bias.setText(parameter.bias)
        self._txt_gain.setText(parameter.gain)
        self._txt_base_level.setText(parameter.base_level)
        self._txt_window.setText(parameter.window)
        self._cb_mode.setCurrentIndex(self._cb_mode.findText(parameter.mode))

    def pulseHeightAnalyser(self):
        return self.parameter()

    def setPulseHeightAnalyser(self, pha):
        self.setParameter(pha)

    pha = pulseHeightAnalyser

    setPha = setPulseHeightAnalyser

    def setReadOnly(self, state):
        ParameterWidget.setReadOnly(self, state)
        self._txt_bias.setReadOnly(state)
        self._txt_gain.setReadOnly(state)
        self._txt_base_level.setReadOnly(state)
        self._txt_window.setReadOnly(state)
        self._cb_mode.setEnabled(not state)

    def isReadOnly(self):
        return ParameterWidget.isReadOnly(self) and \
            self._txt_bias.isReadOnly() and \
            self._txt_gain.isReadOnly() and \
            self._txt_base_level.isReadOnly() and \
            self._txt_window.isReadOnly() and \
            not self._cb_mode.isEnabled()

    def hasAcceptableInput(self):
        return ParameterWidget.hasAcceptableInput(self) and \
            self._txt_bias.hasAcceptableInput() and \
            self._txt_gain.hasAcceptableInput() and \
            self._txt_base_level.hasAcceptableInput() and \
            self._txt_window.hasAcceptableInput()

class WindowWidget(ParameterWidget):

    class _WindowModel(QAbstractTableModel):

        def __init__(self):
            QAbstractTableModel.__init__(self)
            self.layers = []

        def rowCount(self, *args, **kwargs):
            return len(self.layers)

        def columnCount(self, *args, **kwargs):
            return 2

        def data(self, index, role):
            if not index.isValid() or not (0 <= index.row() < len(self.layers)):
                return None
            if role != Qt.DisplayRole:
                return None

            layer = self.layers[index.row()]
            column = index.column()
            if column == 0:
                return layer.material
            elif column == 1:
                return '%s' % layer.thickness

        def headerData(self, section , orientation, role):
            if role != Qt.DisplayRole:
                return None
            if orientation == Qt.Horizontal:
                if section == 0:
                    return 'Material'
                elif section == 1:
                    return 'Thickness'
            elif orientation == Qt.Vertical:
                return str(section + 1)

        def flags(self, index):
            if not index.isValid():
                return Qt.ItemIsEnabled

            return Qt.ItemFlags(QAbstractTableModel.flags(self, index) |
                                Qt.ItemIsEditable)

        def setData(self, index, value, role=Qt.EditRole):
            if not index.isValid() or \
                    not (0 <= index.row() < len(self.layers)):
                return False

            layer = self.layers[index.row()]
            column = index.column()
            if column == 0:
                layer.material = value
            elif column == 1:
                layer.thickness = value

            self.dataChanged.emit(index, index)
            return True

        def insertRows(self, row, count=1, parent=None):
            if count == 0:
                return False
            if parent is None:
                parent = QModelIndex()
            self.beginInsertRows(parent, row, row + count - 1)

            for i in range(count):
                self.layers.insert(row + i, WindowLayer("unknown", 0.0))

            self.endInsertRows()
            return True

        def removeRows(self, row, count=1, parent=None):
            if count == 0:
                return False
            if parent is None:
                parent = QModelIndex()
            self.beginRemoveRows(parent, row, row + count - 1)

            self.layers = self.layers[:row] + self.layers[row + count:]

            self.endRemoveRows()
            return True

    class _WindowDelegate(QItemDelegate):

        def __init__(self, parent=None):
            QItemDelegate.__init__(self, parent)

        def createEditor(self, parent, option, index):
            column = index.column()
            if column == 0:
                return TextAttributeLineEdit(WindowLayer.material, parent)
            elif column == 1:
                return NumericalAttributeLineEdit(WindowLayer.thickness, parent)
            else:
                return QItemDelegate.createEditor(self, parent, option, index)

        def setEditorData(self, editor, index):
            text = index.model().data(index, Qt.DisplayRole)
            column = index.column()
            if column == 0:
                editor.setText(text)
            elif column == 1:
                editor.setText(text)
            else:
                QItemDelegate.setEditorData(self, editor, index)

        def setModelData(self, editor, model, index):
            column = index.column()
            if column == 0:
                model.setData(index, editor.text())
            elif column == 1:
                model.setData(index, editor.text())
            else:
                return QItemDelegate.setModelData(self, editor, model, index)

    def __init__(self, parent=None):
        ParameterWidget.__init__(self, Window, parent)

    def _init_ui(self):
        # Widgets
        model = self._WindowModel()

        self._table = QTableView()
        self._table.setModel(model)
        self._table.setItemDelegate(self._WindowDelegate(self))
        self._table.horizontalHeader().setStretchLastSection(True)

        self._toolbar = QToolBar()
        action_add = self._toolbar.addAction(getIcon("list-add"), "Add layer")
        action_remove = self._toolbar.addAction(getIcon("list-remove"), "Remove layer")

        # Layouts
        layout = ParameterWidget._init_ui(self)
        layout.addRow(self._table)
        layout.addRow(self._toolbar)

        # Signals
        action_add.triggered.connect(self._on_add)
        action_remove.triggered.connect(self._on_remove)

        model.dataChanged.connect(self.edited)
        model.rowsInserted.connect(self.edited)
        model.rowsRemoved.connect(self.edited)

        return layout

    def _on_add(self):
        index = self._table.selectionModel().currentIndex()
        model = self._table.model()
        model.insertRows(index.row() + 1)

    def _on_remove(self):
        selection = self._table.selectionModel().selection().indexes()
        if len(selection) == 0:
            QMessageBox.warning(self, "Window layer", "Select a layer")
            return

        model = self._table.model()
        for row in sorted(map(methodcaller('row'), selection), reverse=True):
            model.removeRow(row)

    def parameter(self, parameter=None):
        parameter = ParameterWidget.parameter(self, parameter)
        parameter.layers.clear()
        for layer in self._table.model().layers:
            parameter.append_layer(layer.material, layer.thickness) # copy
        return parameter

    def setParameter(self, window):
        model = self._table.model()
        model.layers = window.layers
        model.reset()

    def window(self):
        return self.parameter()

    def setWindow(self, window):
        self.setParameter(window)

    def setReadOnly(self, state):
        ParameterWidget.setReadOnly(self, state)
        if state:
            trigger = QTableView.EditTrigger.NoEditTriggers
        else:
            trigger = QTableView.EditTrigger.AllEditTriggers
        self._table.setEditTriggers(trigger)
        self._toolbar.setEnabled(not state)

    def isReadOnly(self):
        return ParameterWidget.isReadOnly(self) and \
            self._table.editTriggers() == QTableView.EditTrigger.NoEditTriggers and \
            not self._toolbar.isEnabled()

class _DetectorWidget(_ConditionWidget):

    def _init_ui(self):
        # Widgets
        self._cb_signal_type = QComboBox()
        self._cb_signal_type.addItems([None] + list(_SIGNAL_TYPES))
        self._txt_manufacturer = TextAttributeLineEdit(self.CLASS.manufacturer)
        self._txt_model = TextAttributeLineEdit(self.CLASS.model)
        self._txt_serial_number = TextAttributeLineEdit(self.CLASS.serial_number)
        self._txt_measurement_unit = UnitAttributeLineEdit(self.CLASS.measurement_unit)
        self._txt_elevation = NumericalAttributeLineEdit(self.CLASS.elevation)
        self._txt_azimuth = NumericalAttributeLineEdit(self.CLASS.azimuth)
        self._txt_distance = NumericalAttributeLineEdit(self.CLASS.distance)
        self._txt_area = NumericalAttributeLineEdit(self.CLASS.area)
        self._txt_solid_angle = NumericalAttributeLineEdit(self.CLASS.solid_angle)
        self._txt_semi_angle = NumericalAttributeLineEdit(self.CLASS.semi_angle)
        self._txt_temperature = NumericalAttributeLineEdit(self.CLASS.temperature)

        # Layout
        layout = _ConditionWidget._init_ui(self)
        layout.addRow('Type of signal', self._cb_signal_type)
        layout.addRow('Manufacturer', self._txt_manufacturer)
        layout.addRow('Model', self._txt_model)
        layout.addRow('Serial number', self._txt_serial_number)
        layout.addRow('Measurement unit', self._txt_measurement_unit)
        layout.addRow('Elevation', self._txt_elevation)
        layout.addRow('Azimuth', self._txt_azimuth)
        layout.addRow('Distance', self._txt_distance)
        layout.addRow('Area', self._txt_area)
        layout.addRow('Solid angle', self._txt_solid_angle)
        layout.addRow('Semi angle', self._txt_semi_angle)
        layout.addRow('Temperature', self._txt_temperature)

        # Signals
        self._cb_signal_type.currentIndexChanged.connect(self.edited)
        self._txt_manufacturer.textEdited.connect(self.edited)
        self._txt_model.textEdited.connect(self.edited)
        self._txt_serial_number.textEdited.connect(self.edited)
        self._txt_measurement_unit.textEdited.connect(self.edited)
        self._txt_elevation.textEdited.connect(self.edited)
        self._txt_azimuth.textEdited.connect(self.edited)
        self._txt_distance.textEdited.connect(self.edited)
        self._txt_area.textEdited.connect(self.edited)
        self._txt_solid_angle.textEdited.connect(self.edited)
        self._txt_semi_angle.textEdited.connect(self.edited)
        self._txt_temperature.textEdited.connect(self.edited)

        return layout

    def parameter(self, parameter=None):
        parameter = _ConditionWidget.parameter(self, parameter)
        parameter.signal_type = self._cb_signal_type.currentText()
        parameter.manufacturer = self._txt_manufacturer.text()
        parameter.model = self._txt_model.text()
        parameter.serial_number = self._txt_serial_number.text()
        parameter.measurement_unit = self._txt_measurement_unit.text()
        parameter.elevation = self._txt_elevation.text()
        parameter.azimuth = self._txt_azimuth.text()
        parameter.distance = self._txt_distance.text()
        parameter.area = self._txt_area.text()
        parameter.solid_angle = self._txt_solid_angle.text()
        parameter.semi_angle = self._txt_semi_angle.text()
        parameter.temperature = self._txt_temperature.text()
        return parameter

    def setParameter(self, condition):
        _ConditionWidget.setParameter(self, condition)
        self._cb_signal_type.setCurrentIndex(self._cb_signal_type.findText(condition.signal_type))
        self._txt_manufacturer.setText(condition.manufacturer)
        self._txt_model.setText(condition.model)
        self._txt_serial_number.setText(condition.serial_number)
        self._txt_measurement_unit.setText(condition.measurement_unit)
        self._txt_elevation.setText(condition.elevation)
        self._txt_azimuth.setText(condition.azimuth)
        self._txt_distance.setText(condition.distance)
        self._txt_area.setText(condition.area)
        self._txt_solid_angle.setText(condition.solid_angle)
        self._txt_semi_angle.setText(condition.semi_angle)
        self._txt_temperature.setText(condition.temperature)

    def setReadOnly(self, state):
        _ConditionWidget.setReadOnly(self, state)
        self._cb_signal_type.setEnabled(not state)
        self._txt_manufacturer.setReadOnly(state)
        self._txt_model.setReadOnly(state)
        self._txt_serial_number.setReadOnly(state)
        self._txt_measurement_unit.setReadOnly(state)
        self._txt_elevation.setReadOnly(state)
        self._txt_azimuth.setReadOnly(state)
        self._txt_distance.setReadOnly(state)
        self._txt_area.setReadOnly(state)
        self._txt_solid_angle.setReadOnly(state)
        self._txt_semi_angle.setReadOnly(state)
        self._txt_temperature.setReadOnly(state)

    def isReadOnly(self):
        return _ConditionWidget.isReadOnly(self) and \
            not self._cb_signal_type.isEnabled() and \
            self._txt_manufacturer.isReadOnly() and \
            self._txt_model.isReadOnly() and \
            self._txt_serial_number.isReadOnly() and \
            self._txt_measurement_unit.isReadOnly() and \
            self._txt_elevation.isReadOnly() and \
            self._txt_azimuth.isReadOnly() and \
            self._txt_distance.isReadOnly() and \
            self._txt_area.isReadOnly() and \
            self._txt_solid_angle.isReadOnly() and \
            self._txt_semi_angle.isReadOnly() and \
            self._txt_temperature.isReadOnly()

    def hasAcceptableInput(self):
        return _ConditionWidget.hasAcceptableInput(self) and \
            self._txt_manufacturer.hasAcceptableInput() and \
            self._txt_model.hasAcceptableInput() and \
            self._txt_serial_number.hasAcceptableInput() and \
            self._txt_measurement_unit.hasAcceptableInput() and \
            self._txt_elevation.hasAcceptableInput() and \
            self._txt_azimuth.hasAcceptableInput() and \
            self._txt_distance.hasAcceptableInput() and \
            self._txt_area.hasAcceptableInput() and \
            self._txt_solid_angle.hasAcceptableInput() and \
            self._txt_semi_angle.hasAcceptableInput() and \
            self._txt_temperature.hasAcceptableInput()

class DetectorCameraWidget(_DetectorWidget):

    def __init__(self, parent=None):
        _DetectorWidget.__init__(self, DetectorCamera, parent)

    def _init_ui(self):
        # Widgets
        self._txt_pixel_count_u = NumericalAttributeLineEdit(self.CLASS.pixel_count_u)
        self._txt_pixel_count_u.setFormat('{0:d}')
        self._txt_pixel_count_v = NumericalAttributeLineEdit(self.CLASS.pixel_count_v)
        self._txt_pixel_count_v.setFormat('{0:d}')
        self._txt_exposure_time = NumericalAttributeLineEdit(self.CLASS.exposure_time)
        self._txt_magnification = NumericalAttributeLineEdit(self.CLASS.magnification)
        self._txt_magnification.setFormat('{0:d}')
        self._txt_focal_length = NumericalAttributeLineEdit(self.CLASS.focal_length)

        # Layout
        layout = _DetectorWidget._init_ui(self)
        layout.insertRow(0, 'Horizontal pixel count', self._txt_pixel_count_u)
        layout.insertRow(1, 'Vertical pixel count', self._txt_pixel_count_v)
        layout.addRow('Exposure time', self._txt_exposure_time)
        layout.addRow('Magnification', self._txt_magnification)
        layout.addRow('Focal length', self._txt_focal_length)

        # Signals
        self._txt_pixel_count_u.textEdited.connect(self.edited)
        self._txt_pixel_count_v.textEdited.connect(self.edited)
        self._txt_exposure_time.textEdited.connect(self.edited)
        self._txt_magnification.textEdited.connect(self.edited)
        self._txt_focal_length.textEdited.connect(self.edited)

        return layout

    def _create_parameter(self):
        return self.CLASS(1, 1)

    def parameter(self, parameter=None):
        parameter = _DetectorWidget.parameter(self, parameter)
        parameter.pixel_count_u = self._txt_pixel_count_u.text()
        parameter.pixel_count_v = self._txt_pixel_count_v.text()
        parameter.exposure_time = self._txt_exposure_time.text()
        parameter.magnification = self._txt_magnification.text()
        parameter.focal_length = self._txt_focal_length.text()
        return parameter

    def setParameter(self, condition):
        _DetectorWidget.setParameter(self, condition)
        self._txt_pixel_count_u.setText(condition.pixel_count_u)
        self._txt_pixel_count_v.setText(condition.pixel_count_v)
        self._txt_exposure_time.setText(condition.exposure_time)
        self._txt_magnification.setText(condition.magnification)
        self._txt_focal_length.setText(condition.focal_length)

    def setReadOnly(self, state):
        _DetectorWidget.setReadOnly(self, state)
        self._txt_pixel_count_u.setReadOnly(state)
        self._txt_pixel_count_v.setReadOnly(state)
        self._txt_exposure_time.setReadOnly(state)
        self._txt_magnification.setReadOnly(state)
        self._txt_focal_length.setReadOnly(state)

    def isReadOnly(self):
        return _DetectorWidget.isReadOnly(self) and \
            self._txt_pixel_count_u.isReadOnly() and \
            self._txt_pixel_count_v.isReadOnly() and \
            self._txt_exposure_time.isReadOnly() and \
            self._txt_magnification.isReadOnly() and \
            self._txt_focal_length.isReadOnly()

    def hasAcceptableInput(self):
        return _DetectorWidget.hasAcceptableInput(self) and \
            self._txt_pixel_count_u.hasAcceptableInput() and \
            self._txt_pixel_count_v.hasAcceptableInput() and \
            self._txt_exposure_time.hasAcceptableInput() and \
            self._txt_magnification.hasAcceptableInput() and \
            self._txt_focal_length.hasAcceptableInput()

class DetectorSpectrometerWidget(_DetectorWidget):

    def __init__(self, parent=None):
        _DetectorWidget.__init__(self, DetectorSpectrometer, parent)

    def _init_ui(self):
        # Widgets
        self._txt_channel_count = NumericalAttributeLineEdit(self.CLASS.channel_count)
        self._txt_channel_count.setFormat('{0:d}')
        self._wdg_calibration = CalibrationWidget()
        self._cb_collection_mode = QComboBox()
        self._cb_collection_mode.addItems([None] + list(_COLLECTION_MODES))

        # Layout
        layout = _DetectorWidget._init_ui(self)
        layout.insertRow(0, '<i>Channel count</i>', self._txt_channel_count)
        layout.insertRow(1, '<i>Calibration</i>', self._wdg_calibration)
        layout.addRow('Collection mode', self._cb_collection_mode)

        # Signals
        self._txt_channel_count.textEdited.connect(self.edited)
        self._wdg_calibration.edited.connect(self.edited)
        self._cb_collection_mode.currentIndexChanged.connect(self.edited)

        return layout

    def _create_parameter(self):
        return self.CLASS(1, CalibrationConstant('Quantity', 'm', 0.0))

    def parameter(self, parameter=None):
        parameter = _DetectorWidget.parameter(self, parameter)
        parameter.channel_count = self._txt_channel_count.text()
        parameter.calibration = self._wdg_calibration.calibration()
        parameter.collection_mode = self._cb_collection_mode.currentText()
        return parameter

    def setParameter(self, condition):
        _DetectorWidget.setParameter(self, condition)
        self._txt_channel_count.setText(condition.channel_count)
        self._wdg_calibration.setCalibration(condition.calibration)
        self._cb_collection_mode.setCurrentIndex(self._cb_collection_mode.findText(condition.collection_mode))

    def setReadOnly(self, state):
        _DetectorWidget.setReadOnly(self, state)
        self._txt_channel_count.setReadOnly(state)
        self._wdg_calibration.setReadOnly(state)
        self._cb_collection_mode.setEnabled(not state)

    def isReadOnly(self):
        return _DetectorWidget.isReadOnly(self) and \
            self._txt_channel_count.isReadOnly() and \
            self._wdg_calibration.isReadOnly() and \
            not self._cb_collection_mode.isEnabled()

    def hasAcceptableInput(self):
        return _DetectorWidget.hasAcceptableInput(self) and \
            self._txt_channel_count.hasAcceptableInput() and \
            self._wdg_calibration.hasAcceptableInput()

class DetectorSpectrometerCLWidget(DetectorSpectrometerWidget):

    def __init__(self, parent=None):
        _DetectorWidget.__init__(self, DetectorSpectrometerCL, parent)

    def _init_ui(self):
        # Widgets
        self._txt_grating_d = NumericalAttributeLineEdit(self.CLASS.grating_d)

        # Layouts
        layout = DetectorSpectrometerWidget._init_ui(self)
        layout.addRow('Grating spacing', self._txt_grating_d)

        # Signals
        self._txt_grating_d.textEdited.connect(self.edited)

        return layout

    def _create_parameter(self):
        return self.CLASS(1, CalibrationConstant('Quantity', 'm', 0.0))

    def parameter(self, parameter=None):
        parameter = DetectorSpectrometerWidget.parameter(self, parameter)
        parameter.grating_d = self._txt_grating_d.text()
        return parameter

    def setParameter(self, condition):
        DetectorSpectrometerWidget.setParameter(self, condition)
        self._txt_grating_d.setText(condition.grating_d)

    def setReadOnly(self, state):
        DetectorSpectrometerWidget.setReadOnly(self, state)
        self._txt_grating_d.setReadOnly(state)

    def isReadOnly(self):
        return DetectorSpectrometerWidget.isReadOnly(self) and \
            self._txt_grating_d.isReadOnly()

    def hasAcceptableInput(self):
        return DetectorSpectrometerWidget.hasAcceptableInput(self) and \
            self._txt_grating_d.hasAcceptableInput()

class DetectorSpectrometerWDSWidget(DetectorSpectrometerWidget):

    def __init__(self, parent=None):
        _DetectorWidget.__init__(self, DetectorSpectrometerWDS, parent)

    def _init_ui(self):
        # Widgets
        self._txt_dispersion_element = TextAttributeLineEdit(self.CLASS.dispersion_element)
        self._txt_crystal_2d = NumericalAttributeLineEdit(self.CLASS.crystal_2d)
        self._txt_rowland_circle_diameter = NumericalAttributeLineEdit(self.CLASS.rowland_circle_diameter)
        self._wdg_pulse_height_analyser = PulseHeightAnalyserWidget()
        self._wdg_window = WindowWidget()

        # Layouts
        layout = DetectorSpectrometerWidget._init_ui(self)
        layout.addRow('Dispersion element', self._txt_dispersion_element)
        layout.addRow('Crystal 2d-spacing', self._txt_crystal_2d)
        layout.addRow('Rowland circle diameter', self._txt_rowland_circle_diameter)
        layout.addRow('Pulse height analyser', self._wdg_pulse_height_analyser)
        layout.addRow('Window', self._wdg_window)

        # Signals
        self._txt_dispersion_element.textEdited.connect(self.edited)
        self._txt_crystal_2d.textEdited.connect(self.edited)
        self._txt_rowland_circle_diameter.textEdited.connect(self.edited)
        self._wdg_pulse_height_analyser.edited.connect(self.edited)
        self._wdg_window.edited.connect(self.edited)

        return layout

    def _create_parameter(self):
        return self.CLASS(1, CalibrationConstant('Quantity', 'm', 0.0))

    def parameter(self, parameter=None):
        parameter = DetectorSpectrometerWidget.parameter(self, parameter)
        parameter.dispersion_element = self._txt_dispersion_element.text()
        parameter.crystal_2d = self._txt_crystal_2d.text()
        parameter.rowland_circle_diameter = self._txt_rowland_circle_diameter.text()
        parameter.pulse_height_analyser = self._wdg_pulse_height_analyser.pha()
        parameter.window = self._wdg_window.window()
        return parameter

    def setParameter(self, condition):
        DetectorSpectrometerWidget.setParameter(self, condition)
        self._txt_dispersion_element.setText(condition.dispersion_element)
        self._txt_crystal_2d.setText(condition.crystal_2d)
        self._txt_rowland_circle_diameter.setText(condition.rowland_circle_diameter)
        self._wdg_pulse_height_analyser.setPha(condition.pulse_height_analyser)
        self._wdg_window.setWindow(condition.window)

    def setReadOnly(self, state):
        DetectorSpectrometerWidget.setReadOnly(self, state)
        self._txt_dispersion_element.setReadOnly(state)
        self._txt_crystal_2d.setReadOnly(state)
        self._txt_rowland_circle_diameter.setReadOnly(state)
        self._wdg_pulse_height_analyser.setReadOnly(state)
        self._wdg_window.setReadOnly(state)

    def isReadOnly(self):
        return DetectorSpectrometerWidget.isReadOnly(self) and \
            self._txt_dispersion_element.isReadOnly() and \
            self._txt_crystal_2d.isReadOnly() and \
            self._txt_rowland_circle_diameter.isReadOnly() and \
            self._wdg_pulse_height_analyser.isReadOnly() and \
            self._wdg_window.isReadOnly()

    def hasAcceptableInput(self):
        return DetectorSpectrometerWidget.hasAcceptableInput(self) and \
            self._txt_dispersion_element.hasAcceptableInput() and \
            self._txt_crystal_2d.hasAcceptableInput() and \
            self._txt_rowland_circle_diameter.hasAcceptableInput() and \
            self._wdg_pulse_height_analyser.hasAcceptableInput() and \
            self._wdg_window.hasAcceptableInput()

class DetectorSpectrometerXEDSWidget(DetectorSpectrometerWidget):

    def __init__(self, parent=None):
        _DetectorWidget.__init__(self, DetectorSpectrometerXEDS, parent)

    def _init_ui(self):
        # Widgets
        self._cb_technology = QComboBox()
        self._cb_technology.addItems([None] + list(_XEDS_TECHNOLOGIES))
        self._txt_nominal_throughput = NumericalAttributeLineEdit(self.CLASS.nominal_throughput)
        self._txt_time_constant = NumericalAttributeLineEdit(self.CLASS.time_constant)
        self._txt_strobe_rate = NumericalAttributeLineEdit(self.CLASS.strobe_rate)
        self._wdg_window = WindowWidget()

        # Layout
        form = DetectorSpectrometerWidget._init_ui(self)
        form.addRow('Technology', self._cb_technology)
        form.addRow('Nominal throughput', self._txt_nominal_throughput)
        form.addRow('Time constant', self._txt_time_constant)
        form.addRow('Strobe rate', self._txt_strobe_rate)
        form.addRow('Window', self._wdg_window)

        # Signals
        self._cb_technology.currentIndexChanged.connect(self.edited)
        self._txt_nominal_throughput.textEdited.connect(self.edited)
        self._txt_time_constant.textEdited.connect(self.edited)
        self._txt_strobe_rate.textEdited.connect(self.edited)
        self._wdg_window.edited.connect(self.edited)

        return form

    def _create_parameter(self):
        return self.CLASS(1, CalibrationConstant('Quantity', 'm', 0.0))

    def parameter(self, parameter=None):
        parameter = DetectorSpectrometerWidget.parameter(self, parameter)
        parameter.technology = self._cb_technology.currentText()
        parameter.nominal_throughput = self._txt_nominal_throughput.text()
        parameter.time_constant = self._txt_time_constant.text()
        parameter.strobe_rate = self._txt_strobe_rate.text()
        parameter.window = self._wdg_window.window()
        return parameter

    def setParameter(self, condition):
        DetectorSpectrometerWidget.setParameter(self, condition)
        self._cb_technology.setCurrentIndex(self._cb_technology.findText(condition.technology))
        self._txt_nominal_throughput.setText(condition.nominal_throughput)
        self._txt_time_constant.setText(condition.time_constant)
        self._txt_strobe_rate.setText(condition.strobe_rate)
        self._wdg_window.setWindow(condition.window)

    def setReadOnly(self, state):
        DetectorSpectrometerWidget.setReadOnly(self, state)
        self._cb_technology.setEnabled(not state)
        self._txt_nominal_throughput.setReadOnly(state)
        self._txt_time_constant.setReadOnly(state)
        self._txt_strobe_rate.setReadOnly(state)
        self._wdg_window.setReadOnly(state)

    def isReadOnly(self):
        return DetectorSpectrometerWidget.isReadOnly(self) and \
            not self._cb_technology.isEnabled() and \
            self._txt_nominal_throughput.isReadOnly() and \
            self._txt_time_constant.isReadOnly() and \
            self._txt_strobe_rate.isReadOnly() and \
            self._wdg_window.isReadOnly()

    def hasAcceptableInput(self):
        return DetectorSpectrometerWidget.hasAcceptableInput(self) and \
            self._txt_nominal_throughput.hasAcceptableInput() and \
            self._txt_time_constant.hasAcceptableInput() and \
            self._txt_strobe_rate.hasAcceptableInput() and \
            self._wdg_window.hasAcceptableInput()



"""
Add scale bar to matplotlib's image 
"""

# Standard library modules.
import bisect
from operator import itemgetter

# Third party modules.
from matplotlib.artist import Artist
from matplotlib.cbook import is_string_like

from mpl_toolkits.axes_grid.anchored_artists import AnchoredSizeBar

# Local modules.
from pyhmsa.type.unit import _PREFIXES_VALUES

# Globals and constants variables.

class ScaleBar(Artist):

    zorder = 5

    _PREFERRED_VALUES = [1, 2, 5, 10, 15, 20, 25, 50, 75, 100, 125, 150, 200, 500, 750]

    _LOCATIONS = {'upper right':  1,
                  'upper left':   2,
                  'lower left':   3,
                  'lower right':  4,
                  'right':        5,
                  'center left':  6,
                  'center right': 7,
                  'lower center': 8,
                  'upper center': 9,
                  'center':       10,
              }

    def __init__(self, dx_m, length_fraction=0.2, height_fraction=0.01,
                 location=1, pad=0.2, border_pad=0.1, sep=5, frameon=True,
                 color='b', box_color='w', box_alpha=1.0,
                 label_top=False, font_properties=None,
                 **kwargs):
        """
        Creates a new scale bar.
        
        :arg dx_m: dimension of one pixel in meters (m)
        :arg length_fraction: length of the scale bar as a fraction of the 
            axes's width
        :arg height_fraction: height of the scale bar as a fraction of the 
            axes's height
        :arg location: a location code (same as legend)
        :arg pad: fraction of the legend font size
        :arg border_pad : fraction of the legend font size
        :arg sep : separation between scale bar and label in points
        :arg frameon : if True, will draw a box around the horizontal bar and label
        :arg color : color for the size bar and label
        :arg box_color: color of the box (if *frameon*)
        :arg box_alpha: transparency of box
        :arg label_top : if True, the label will be over the rectangle
        :arg font_properties: a matplotlib.font_manager.FontProperties instance, optional
            sets the font properties for the label text
        """
        Artist.__init__(self)

        self.dx_m = dx_m
        self.length_fraction = length_fraction
        self.height_fraction = height_fraction
        self.location = location
        self.pad = pad
        self.border_pad = border_pad
        self.sep = sep
        self.frameon = frameon
        self.color = color
        self.box_color = box_color
        self.box_alpha = box_alpha
        self.label_top = label_top
        self.font_properties = font_properties
        self._kwargs = kwargs

    def _calculate_length(self, length_px):
        length_m = length_px * self._dx_m

        prefixes_values = _PREFIXES_VALUES.copy()
        prefixes_values[''] = 1.0
        prefixes_values.pop('u')
        prefixes_values = sorted(prefixes_values.items(), key=itemgetter(1))
        values = [prefix_value[1] for prefix_value in prefixes_values]
        index = bisect.bisect_left(values, length_m)
        unit, factor = prefixes_values[index - 1]

        length_unit = length_m / factor
        index = bisect.bisect_left(self._PREFERRED_VALUES, length_unit)
        length_unit = self._PREFERRED_VALUES[index - 1]

        length_px = length_unit * factor / self._dx_m
        label = '%i %sm' % (length_unit, unit)

        return length_px, label

    def draw(self, renderer, *args, **kwargs):
        if not self.get_visible():
            return

        ax = self.get_axes()
        xlim, ylim = ax.get_xlim(), ax.get_ylim()

        length_px = abs(xlim[1] - xlim[0]) * self.length_fraction
        length_px, label = self._calculate_length(length_px)

        size_vertical = abs(ylim[1] - ylim[0]) * self.height_fraction

        sizebar = AnchoredSizeBar(transform=ax.transData,
                                  size=length_px,
                                  label=label,
                                  loc=self.location,
                                  pad=self.pad,
                                  borderpad=self.border_pad,
                                  sep=self.sep,
                                  frameon=self.frameon,
                                  size_vertical=size_vertical,
                                  color=self.color,
                                  label_top=self.label_top,
                                  fontproperties=self.font_properties,
                                  **self._kwargs)
        sizebar.set_axes(ax)
        sizebar.set_figure(self.get_figure())
        sizebar.patch.set_color(self.box_color)
        sizebar.patch.set_alpha(self.box_alpha)
        sizebar.draw(renderer)

    @property
    def dx_m(self):
        return self._dx_m

    @dx_m.setter
    def dx_m(self, dx_m):
        self._dx_m = float(dx_m)

    @property
    def length_fraction(self):
        return self._length_fraction

    @length_fraction.setter
    def length_fraction(self, fraction):
        assert 0.0 <= fraction <= 1.0
        self._length_fraction = float(fraction)

    @property
    def height_fraction(self):
        return self._height_fraction

    @height_fraction.setter
    def height_fraction(self, fraction):
        assert 0.0 <= fraction <= 1.0
        self._height_fraction = float(fraction)

    @property
    def location(self):
        return self._location

    @location.setter
    def location(self, loc):
        if is_string_like(loc):
            if loc not in self._LOCATIONS:
                raise ValueError('Unknown location code: %s' % loc)
            loc = self._LOCATIONS[loc]
        self._location = loc

    @property
    def pad(self):
        return self._pad

    @pad.setter
    def pad(self, pad):
        self._pad = float(pad)

    @property
    def border_pad(self):
        return self._border_pad

    @border_pad.setter
    def border_pad(self, pad):
        self._border_pad = float(pad)

    @property
    def sep(self):
        return self._sep

    @sep.setter
    def sep(self, sep):
        self._sep = float(sep)

    @property
    def frameon(self):
        return self._frameon

    @frameon.setter
    def frameon(self, on):
        self._frameon = bool(on)

    @property
    def color(self):
        return self._color

    @color.setter
    def color(self, color):
        self._color = color

    @property
    def box_color(self):
        return self._box_color

    @box_color.setter
    def box_color(self, color):
        self._box_color = color

    @property
    def box_alpha(self):
        return self._box_alpha

    @box_alpha.setter
    def box_alpha(self, alpha):
        assert 0.0 <= alpha <= 1.0
        self._box_alpha = alpha

    @property
    def label_top(self):
        return self._label_top

    @label_top.setter
    def label_top(self, top):
        self._label_top = bool(top)

    @property
    def font_properties(self):
        return self._font_properties

    @font_properties.setter
    def font_properties(self, props):
        self._font_properties = props




"""
Widgets for parameters and attributes
"""

# Standard library modules.

# Third party modules.
from PySide.QtGui import \
    (QWidget, QLineEdit, QRegExpValidator, QValidator, QPushButton,
     QVBoxLayout, QFormLayout)
from PySide.QtCore import QRegExp, Signal

import six

# Local modules.
from pyhmsa.type.numerical import convert_value
from pyhmsa.type.unit import validate_unit
from pyhmsa.util.element_properties import get_symbol, get_atomic_number

from pyhmsa.gui.util.periodictable import PeriodicTableDialog

from pyhmsa.gui.util.human import camelcase_to_words

# Globals and constants variables.

class _AttributeMixin(object):

    def __init__(self, attribute):
        self._attribute = attribute

        self.setAccessibleName(attribute.name)
        self.setToolTip(attribute.__doc__.title())

class _AttributeLineEdit(QLineEdit, _AttributeMixin):

    def __init__(self, attribute, *args, **kwargs):
        QLineEdit.__init__(self, *args, **kwargs)
        _AttributeMixin.__init__(self, attribute)

        if attribute.is_required():
            pattern = QRegExp(r"^(?!\s*$).+")
            validator = QRegExpValidator(pattern)
            self.setValidator(validator)

        self.textChanged.connect(self._onTextChanged)
        self.textChanged.emit('')

    def _onTextChanged(self):
        if self.hasAcceptableInput():
            self.setStyleSheet("background: none")
        else:
            self.setStyleSheet("background: pink")

class TextAttributeLineEdit(_AttributeLineEdit):

    def text(self):
        text = _AttributeLineEdit.text(self)
        if len(text.strip()) == 0:
            return None
        return text

    def setText(self, text):
        if text is None:
            text = ''
        return _AttributeLineEdit.setText(self, text)

class NumericalAttributeLineEdit(_AttributeLineEdit):

    class _Validator(QValidator):

        def __init__(self, attribute):
            QValidator.__init__(self)
            self._attribute = attribute

        def validate(self, text, pos):
            parts = text.split()
            if len(parts) == 0:
                if self._attribute.is_required():
                    return QValidator.Intermediate
                else:
                    return QValidator.Acceptable

            elif len(parts) == 1:
                try:
                    float(parts[0])
                except ValueError:
                    return QValidator.Intermediate
                else:
                    return QValidator.Acceptable

            elif len(parts) == 2:
                try:
                    float(parts[0])
                    validate_unit(parts[1])
                except ValueError:
                    return QValidator.Intermediate
                else:
                    return QValidator.Acceptable

            else:
                return QValidator.Invalid

        def fixup(self, text):
            return text

    def __init__(self, attribute, *args, **kwargs):
        _AttributeLineEdit.__init__(self, attribute, *args, **kwargs)

        self._format = '{0:g}'

        self.setValidator(self._Validator(attribute))

        tooltip = attribute.__doc__.title()
        if attribute.default_unit is not None:
            tooltip += ' (default unit: %s)' % attribute.default_unit
        self.setToolTip(tooltip)

        self.editingFinished.connect(self._onEditingFinished)

    def _onEditingFinished(self):
        if not self.hasAcceptableInput():
            return
        self.setText(self.text())

    def _parse(self, text):
        parts = text.split()
        if len(parts) == 0:
            value = unit = None
        elif len(parts) == 1:
            value = float(parts[0])
            unit = None
        elif len(parts) == 2:
            value = float(parts[0])
            unit = parts[1]

        return convert_value(value, unit or self._attribute.default_unit)

    def format(self):
        return self._format

    def setFormat(self, fmt):
        self._format = fmt

    def text(self):
        if not self.hasAcceptableInput():
            raise ValueError('Invalid text')
        return self._parse(_AttributeLineEdit.text(self))

    def setText(self, value):
        if isinstance(value, six.string_types):
            value = self._parse(value)

        if value is None:
            return _AttributeLineEdit.setText(self, '')

        text = self._format.format(value)
        unit = getattr(value, 'unit', None)
        if unit is not None:
            text += ' ' + unit

        return _AttributeLineEdit.setText(self, text)

class UnitAttributeLineEdit(_AttributeLineEdit):

    class _Validator(QValidator):

        def __init__(self, attribute, valid_units=None):
            QValidator.__init__(self)
            self._attribute = attribute
            self._valid_units = valid_units

        def validate(self, text, pos):
            if not text:
                if self._attribute.is_required():
                    return QValidator.Intermediate
                else:
                    return QValidator.Acceptable

            try:
                validate_unit(text)
            except ValueError:
                return QValidator.Intermediate

            if self._valid_units is not None and \
                    text not in self._valid_units:
                return QValidator.Intermediate

            return QValidator.Acceptable

        def fixup(self, text):
            return text

    def __init__(self, attribute, valid_units=None, *args, **kwargs):
        _AttributeLineEdit.__init__(self, attribute, *args, **kwargs)

        self.setValidator(self._Validator(attribute, valid_units))

    def text(self):
        text = _AttributeLineEdit.text(self)
        if len(text.strip()) == 0:
            return None
        return text

    def setText(self, text):
        if text is None:
            text = ''
        return _AttributeLineEdit.setText(self, text)

class AtomicNumberAttributePushButton(QPushButton, _AttributeMixin):

    selectionChanged = Signal()

    def __init__(self, attribute, *args, **kwargs):
        QPushButton.__init__(self, *args, **kwargs)
        _AttributeMixin.__init__(self, attribute)

        self.setText(str(None))

        self.clicked.connect(self._onClick)

    def _onClick(self):
        dialog = PeriodicTableDialog(self.parent())
        dialog.setMultipleSelection(False)
        dialog.setRequiresSelection(False)

        symbol = self.text()
        if symbol == str(None):
            symbol = None
        dialog.setSelection(symbol)

        if dialog.exec_():
            newsymbol = str(dialog.selectionSymbol())
            self.setText(newsymbol)

            if newsymbol != symbol:
                self.selectionChanged.emit()

    def hasAcceptableInput(self):
        return self.text() != str(None)

    def setAtomicNumber(self, z):
        self.setText(get_symbol(z))

    def atomicNumber(self):
        symbol = self.text()
        if symbol == str(None):
            return None
        return get_atomic_number(symbol)

class ParameterWidget(QWidget):

    edited = Signal()

    def __init__(self, clasz, parent=None):
        QWidget.__init__(self, parent)
        self.setAccessibleName(' '.join(camelcase_to_words(clasz.__name__)))

        # Variable
        self._class = clasz

        # Layout
        layout = QVBoxLayout()
        layout.addLayout(self._init_ui()) # Initialize widgets
        layout.addStretch()
        self.setLayout(layout)

    def _init_ui(self):
        layout = QFormLayout()
        layout.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.ExpandingFieldsGrow) # Fix for Mac OS
        return layout

    def _create_parameter(self):
        return self.CLASS()

    def isReadOnly(self):
        return False

    def setReadOnly(self, state):
        pass

    def hasAcceptableInput(self):
        return True

    def parameter(self, parameter=None):
        if parameter is None:
            parameter = self._create_parameter()
        return parameter

    def setParameter(self, parameter):
        pass

    @property
    def CLASS(self):
        return self._class





#!/usr/bin/env python

# Standard library modules.
import os
import sys
import glob
from distutils.core import Command
from subprocess import check_call

# Third party modules.
from setuptools import setup, find_packages

# Local modules.

# Globals and constants variables.
BASEDIR = os.path.abspath(os.path.dirname(__file__))

class _bdist_fpm(Command):

    description = 'Build using fpm (Effing Package Management)'

    user_options = [('dist-dir=', 'd',
                     "directory to put final built distributions in "
                     "[default: dist]"), ]

    def initialize_options(self):
        self.dist_dir = None

    def finalize_options(self):
        if self.dist_dir is None:
            self.dist_dir = "dist"

    def _run(self, target):
        setup_filepath = os.path.join(BASEDIR, 'setup.py')

        python_bin = 'python3' if sys.version_info.major == 3 else 'python'
        version = '%i.%i' % (sys.version_info.major, sys.version_info.minor)

        args = ['fpm',
                '-s', 'python',
                '-t', target,
                '--force',
                '--verbose',
                '--maintainer', 'Philippe Pinard <philippe.pinard@gmail.com>',
                '--category', 'science',
                '--depends', "%s >= %s" % (python_bin, version),
                '--depends', python_bin + '-numpy',
                '--depends', python_bin + '-six',
                '--depends', python_bin + '-matplotlib',
                '--depends', python_bin + '-pyside',
                '--depends', python_bin + '-hmsa',
                '--no-python-dependencies',
                '--python-bin', python_bin,
                '--name', python_bin + '-hmsa-gui',
                setup_filepath]
        check_call(args)

        self.mkpath(self.dist_dir)
        for srcfilepath in glob.glob('*.%s' % target):
            self.move_file(srcfilepath, self.dist_dir)

class bdist_deb(_bdist_fpm):

    description = 'Build deb '

    def run(self):
        self._run('deb')

with open(os.path.join(BASEDIR, 'VERSION')) as version_file:
    version = version_file.read().strip()

# Get the long description from the relevant file
with open('README.rst', 'r') as f:
    long_description = f.read()

setup(name='pyHMSA-gui',
      version=version,
      description='Graphical components to represent HMSA specification',
      long_description=long_description,

      author='Philippe Pinard',
      author_email='philippe.pinard@gmail.com',
      maintainer='Philippe Pinard',
      maintainer_email='philippe.pinard@gmail.com',

      url='http://pyhmsa.readthedocs.org',
      license='MIT',
      keywords='microscopy microanalysis hmsa file format gui',

      classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Scientific/Engineering :: Physics',
        ],

      packages=find_packages(),
      package_data={'pyhmsa.gui.util': ['icons/*.rcc']},

      install_requires=['pyHMSA', 'PySide', 'matplotlib', 'numpy', 'six'],

      zip_safe=True,

      entry_points=\
        {'pyhmsa.gui.spec.condition.calibration':
            ['CalibrationConstant = pyhmsa.gui.spec.condition.calibration:CalibrationConstantWidget',
             'CalibrationLinear = pyhmsa.gui.spec.condition.calibration:CalibrationLinearWidget',
             'CalibrationPolynomial = pyhmsa.gui.spec.condition.calibration:CalibrationPolynomialWidget',
             'CalibrationExplicit = pyhmsa.gui.spec.condition.calibration:CalibrationExplicitWidget'],
         'pyhmsa.gui.spec.condition':
            ['AcquisitionPoint = pyhmsa.gui.spec.condition.acquisition:AcquisitionPointWidget',
             'AcquisitionMultipoint = pyhmsa.gui.spec.condition.acquisition:AcquisitionMultipointWidget',
             'AcquisitionRasterLinescan = pyhmsa.gui.spec.condition.acquisition:AcquisitionRasterLinescanWidget',
             'AcquisitionRasterXY = pyhmsa.gui.spec.condition.acquisition:AcquisitionRasterXYWidget',
             'AcquisitionRasterXYZ = pyhmsa.gui.spec.condition.acquisition:AcquisitionRasterXYZWidget',

             'DetectorCamera = pyhmsa.gui.spec.condition.detector:DetectorCameraWidget',
             'DetectorSpectrometer = pyhmsa.gui.spec.condition.detector:DetectorSpectrometerWidget',
             'DetectorSpectrometerCL = pyhmsa.gui.spec.condition.detector:DetectorSpectrometerCLWidget',
             'DetectorSpectrometerWDS = pyhmsa.gui.spec.condition.detector:DetectorSpectrometerWDSWidget',
             'DetectorSpectrometerXEDS = pyhmsa.gui.spec.condition.detector:DetectorSpectrometerXEDSWidget',

             'ElementalID = pyhmsa.gui.spec.condition.elementalid:ElementalIDWidget',
             'ElementalIDXray = pyhmsa.gui.spec.condition.elementalid:ElementalIDXrayWidget',

             'Instrument = pyhmsa.gui.spec.condition.instrument:InstrumentWidget',

             'ProbeEM = pyhmsa.gui.spec.condition.probe:ProbeEMWidget',
             'ProbeTEM = pyhmsa.gui.spec.condition.probe:ProbeTEMWidget',

             'RegionOfInterest = pyhmsa.gui.spec.condition.region:RegionOfInterestWidget',

             'SpecimenPosition = pyhmsa.gui.spec.condition.specimenposition:SpecimenPositionWidget',

             'CompositionElemental = pyhmsa.gui.spec.condition.composition:CompositionElementalWidget',

             'Specimen = pyhmsa.gui.spec.condition.specimen:SpecimenWidget',
    #            'SpecimenMultilayer = pyhmsa.gui.spec.condition.specimen:SpecimenMultilayerWidget'
            ],
         'pyhmsa.gui.spec.datum':
            ['Analysis0D.Table = pyhmsa.gui.spec.datum.analysis:Analysis0DTableWidget',
             'Analysis1D.Table = pyhmsa.gui.spec.datum.analysis:Analysis1DTableWidget',
             'Analysis1D.Graph = pyhmsa.gui.spec.datum.analysis:Analysis1DGraphWidget',
             'Analysis2D.Table = pyhmsa.gui.spec.datum.analysis:Analysis2DTableWidget',
             'Analysis2D.Graph = pyhmsa.gui.spec.datum.analysis:Analysis2DGraphWidget',

             'AnalysisList0D.Table = pyhmsa.gui.spec.datum.analysislist:AnalysisList0DTableWidget',
             'AnalysisList1D.Table = pyhmsa.gui.spec.datum.analysislist:AnalysisList1DTableWidget',
             'AnalysisList1D.Graph = pyhmsa.gui.spec.datum.analysislist:AnalysisList1DGraphWidget',
             'AnalysisList2D.Table = pyhmsa.gui.spec.datum.analysislist:AnalysisList2DTableWidget',
             'AnalysisList2D.Graph = pyhmsa.gui.spec.datum.analysislist:AnalysisList2DGraphWidget',

             'ImageRaster2D.Table = pyhmsa.gui.spec.datum.imageraster:ImageRaster2DTableWidget',
             'ImageRaster2D.Graph = pyhmsa.gui.spec.datum.imageraster:ImageRaster2DGraphWidget',
             'ImageRaster2DSpectral.Table = pyhmsa.gui.spec.datum.imageraster:ImageRaster2DSpectralTableWidget',
             'ImageRaster2DSpectral.Graph = pyhmsa.gui.spec.datum.imageraster:ImageRaster2DSpectralGraphWidget',
             ],
         },

      cmdclass={'bdist_deb': bdist_deb},
     )



"""
Base widget for conditions
"""

# Standard library modules.

# Third party modules.

# Local modules.
from pyhmsa.gui.util.parameter import ParameterWidget

# Globals and constants variables.

class _ConditionWidget(ParameterWidget):

    def __init__(self, clasz, parent=None):
        ParameterWidget.__init__(self, clasz, parent)

        name = clasz.TEMPLATE
        if clasz.CLASS is not None:
            name += ' (%s)' % clasz.CLASS
        self.setAccessibleName(name)

    def condition(self, condition=None):
        return self.parameter(condition)

    def setCondition(self, condition):
        self.setParameter(condition)



"""
Convert code to human readable text
"""

# Standard library modules.
import re

# Third party modules.

# Local modules.

# Globals and constants variables.
_CAMELCASE_TO_WORDS_PATTERN = re.compile('([A-Z][a-z0-9]*)')

def camelcase_to_words(text):
    words = _CAMELCASE_TO_WORDS_PATTERN.split(text)
    return [word for word in words if word]


"""
Validation utilities.
"""

# Standard library modules.

# Third party modules.
from PySide.QtGui import QMessageBox

# Local modules.

# Globals and constants variables.

def validate_widget(*widgets):
    invalids = []
    for widget in widgets:
        _validate_widget_recursive(widget, invalids)

    if not invalids:
        return True

    if len(invalids) >= 2:
        message = 'The following fields contain error(s):\n'
        title = 'Validation errors'
    else:
        message = 'The following field contains error(s):\n'
        title = 'Validation error'

    for invalid_widget in invalids:
        message += '- %s\n' % invalid_widget.accessibleName()

    QMessageBox.critical(widget.parent(), title, message)
    return False

def _validate_widget_recursive(widget, invalids):
    if hasattr(widget, 'hasAcceptableInput') and \
            not widget.hasAcceptableInput() and \
            widget.isVisible():
        invalids.append(widget)

    for child in widget.children():
        _validate_widget_recursive(child, invalids)



"""
Elemend ID widget
"""

# Standard library modules.

# Third party modules.

# Local modules.
from pyhmsa.gui.spec.condition.condition import _ConditionWidget
from pyhmsa.gui.util.parameter import \
    (AtomicNumberAttributePushButton, TextAttributeLineEdit,
     NumericalAttributeLineEdit)

from pyhmsa.spec.condition.elementalid import ElementalID, ElementalIDXray

# Globals and constants variables.

class ElementalIDWidget(_ConditionWidget):

    def __init__(self, parent=None):
        _ConditionWidget.__init__(self, ElementalID, parent)

    def _init_ui(self):
        # Widgets
        self._btn_atomic_number = AtomicNumberAttributePushButton(self.CLASS.atomic_number)

        # Layouts
        layout = _ConditionWidget._init_ui(self)
        layout.addRow('<i>Element</i>', self._btn_atomic_number)

        # Signals
        self._btn_atomic_number.selectionChanged.connect(self.edited)

        return layout

    def _create_parameter(self):
        return self.CLASS(1)

    def parameter(self, parameter=None):
        parameter = _ConditionWidget.parameter(self, parameter)
        parameter.atomic_number = self._btn_atomic_number.atomicNumber()
        return parameter

    def setParameter(self, condition):
        _ConditionWidget.setParameter(self, condition)
        self._btn_atomic_number.setAtomicNumber(condition.atomic_number)

    def setReadOnly(self, state):
        _ConditionWidget.setReadOnly(self, state)
        self._btn_atomic_number.setEnabled(not state)

    def isReadOnly(self):
        return _ConditionWidget.isReadOnly(self) and \
            not self._btn_atomic_number.isEnabled()

class ElementalIDXrayWidget(ElementalIDWidget):

    def __init__(self, parent=None):
        _ConditionWidget.__init__(self, ElementalIDXray, parent)

    def _init_ui(self):
        # Widgets
        self._txt_line = TextAttributeLineEdit(self.CLASS.line)
        self._txt_energy = NumericalAttributeLineEdit(self.CLASS.energy)

        # Layouts
        layout = ElementalIDWidget._init_ui(self)
        layout.addRow('<i>Line</i>', self._txt_line)
        layout.addRow('Energy', self._txt_energy)

        # Signals
        self._txt_line.textEdited.connect(self.edited)
        self._txt_energy.textEdited.connect(self.edited)

        return layout

    def _create_parameter(self):
        return self.CLASS(1, 'Ka')

    def parameter(self, parameter=None):
        parameter = ElementalIDWidget.parameter(self, parameter)
        parameter.line = self._txt_line.text()
        parameter.energy = self._txt_energy.text()
        return parameter

    def setParameter(self, condition):
        ElementalIDWidget.setParameter(self, condition)
        self._txt_line.setText(condition.line)
        self._txt_energy.setText(condition.energy)

    def setReadOnly(self, state):
        ElementalIDWidget.setReadOnly(self, state)
        self._txt_line.setReadOnly(state)
        self._txt_energy.setReadOnly(state)

    def isReadOnly(self):
        return ElementalIDWidget.isReadOnly(self) and \
            self._txt_line.isReadOnly() and \
            self._txt_energy.isReadOnly()

    def hasAcceptableInput(self):
        return ElementalIDWidget.hasAcceptableInput(self) and \
            self._txt_line.hasAcceptableInput() and \
            self._txt_energy.hasAcceptableInput()



"""
Analysis widgets
"""

# Standard library modules.

# Third party modules.
from PySide.QtCore import Qt, QAbstractTableModel

from matplotlib.figure import Figure

# Local modules.
from pyhmsa.gui.spec.datum.datum import _DatumTableWidget, _DatumFigureWidget
from pyhmsa.gui.util.mpl.toolbar import NavigationToolbarQT, NavigationToolbarSnapMixinQT
from pyhmsa.gui.util.mpl.modest_image import imshow as modest_imshow

from pyhmsa.spec.datum.analysis import Analysis0D, Analysis1D, Analysis2D
from pyhmsa.spec.condition.detector import DetectorSpectrometer

# Globals and constants variables.

class _Analysis1DNagivationToolbarQT(NavigationToolbarSnapMixinQT,
                                     NavigationToolbarQT):

    def __init__(self, canvas, parent, coordinates=True):
        NavigationToolbarQT.__init__(self, canvas, parent, coordinates)
        NavigationToolbarSnapMixinQT.__init__(self)

    def _init_toolbar(self):
        NavigationToolbarQT._init_toolbar(self)
        NavigationToolbarSnapMixinQT._init_toolbar(self)

class Analysis0DTableWidget(_DatumTableWidget):

    class _TableModel(QAbstractTableModel):

        def __init__(self, datum):
            QAbstractTableModel.__init__(self)
            self._datum = datum

        def rowCount(self, parent=None):
            return 1

        def columnCount(self, parent=None):
            return 1

        def data(self, index, role):
            if not index.isValid() or not (0 <= index.row() < 1):
                return None
            if role != Qt.DisplayRole:
                return None

            return str(self._datum)

        def headerData(self, section , orientation, role):
            if role != Qt.DisplayRole:
                return None
            if orientation == Qt.Horizontal:
                return 'Value'
            elif orientation == Qt.Vertical:
                return str(section + 1)

    def __init__(self, controller, datum=None, parent=None):
        _DatumTableWidget.__init__(self, Analysis0D, controller, datum, parent)

    def _create_model(self, datum):
        return self._TableModel(datum)

class Analysis1DTableWidget(_DatumTableWidget):

    class _TableModel(QAbstractTableModel):

        def __init__(self, datum):
            QAbstractTableModel.__init__(self)
            self._datum = datum

            conditions = datum.conditions.findvalues(DetectorSpectrometer)
            if conditions:
                self._calibration = next(iter(conditions)).calibration
            else:
                self._calibration = None

        def rowCount(self, parent=None):
            return self._datum.channels

        def columnCount(self, parent=None):
            return 2 if self._calibration is not None else 1

        def data(self, index, role):
            if not index.isValid() or not (0 <= index.row() < self._datum.channels):
                return None
            if role != Qt.DisplayRole:
                return None

            row = index.row()
            column = index.column()
            if self._calibration is not None:
                if column == 0:
                    return str(self._calibration(row))
                elif column == 1:
                    return str(self._datum[row])
            else:
                return str(self._datum[row])

        def headerData(self, section , orientation, role):
            if role != Qt.DisplayRole:
                return None
            if orientation == Qt.Horizontal:
                if self._calibration is not None:
                    if section == 0:
                        return '%s (%s)' % (self._calibration.quantity,
                                            self._calibration.unit)
                    elif section == 1:
                        return 'Value'
                else:
                    return 'Value'
            elif orientation == Qt.Vertical:
                return str(section + 1)

    def __init__(self, controller, datum=None, parent=None):
        _DatumTableWidget.__init__(self, Analysis1D, controller, datum, parent)

    def _create_model(self, datum):
        return self._TableModel(datum)

class Analysis1DGraphWidget(_DatumFigureWidget):

    def __init__(self, controller, datum=None, parent=None):
        _DatumFigureWidget.__init__(self, Analysis1D, controller, datum, parent)

    def _create_figure(self):
        fig = Figure((5.0, 4.0), dpi=100)
        self._ax = None
        self._artist = None
        return fig

    def _create_toolbar(self, canvas):
        return _Analysis1DNagivationToolbarQT(canvas, self.parent())

    def _draw_figure(self, fig, datum):
        if datum is None:
            self._ax = None
            self._artist = None
            return

        # Extract data and labels
        xy = datum.get_xy()
        xlabel = datum.get_xlabel()
        ylabel = datum.get_xlabel()

        # Draw
        if self._artist is None:
            self._ax = fig.add_subplot("111")
            self._artist, = self._ax.plot(xy[:, 0], xy[:, 1], zorder=1)
        else:
            self._artist.set_data((xy[:, 0], xy[:, 1]))
            self._ax.relim()
            self._ax.autoscale(tight=True)

        self._ax.set_xlabel(xlabel)
        self._ax.set_ylabel(ylabel)

class Analysis2DTableWidget(_DatumTableWidget):

    class _TableModel(QAbstractTableModel):

        def __init__(self, datum):
            QAbstractTableModel.__init__(self)
            self._datum = datum

        def rowCount(self, parent=None):
            return self._datum.v

        def columnCount(self, parent=None):
            return self._datum.u

        def data(self, index, role):
            if not index.isValid() or \
                    not (0 <= index.row() < self._datum.v) or \
                    not (0 <= index.column() < self._datum.u):
                return None
            if role != Qt.DisplayRole:
                return None

            row = index.row()
            column = index.column()
            return str(self._datum[column, row])

        def headerData(self, section , orientation, role):
            if role != Qt.DisplayRole:
                return None
            if orientation == Qt.Horizontal:
                return str(section + 1)
            elif orientation == Qt.Vertical:
                return str(section + 1)

    def __init__(self, controller, datum=None, parent=None):
        _DatumTableWidget.__init__(self, Analysis2D, controller, datum, parent)

    def _create_model(self, datum):
        return self._TableModel(datum)

class Analysis2DGraphWidget(_DatumFigureWidget):

    def __init__(self, controller, datum=None, parent=None):
        _DatumFigureWidget.__init__(self, Analysis2D, controller, datum, parent)

    def _create_figure(self):
        fig = Figure((5.0, 4.0), dpi=100)
        self._ax = None
        self._artist = None
        return fig

    def _draw_figure(self, fig, datum):
        if self._artist is None: # First draw
            self._ax = fig.add_axes([0.0, 0.0, 1.0, 1.0])
            self._ax.xaxis.set_visible(False)
            self._ax.yaxis.set_visible(False)
            self._artist = modest_imshow(self._ax, datum.T)
            fig.colorbar(self._artist, shrink=0.8)
        else:
            self._artist.set_data(datum.T)
            self._artist.autoscale()



"""
Analysis list widgets
"""

# Standard library modules.

# Third party modules.
from PySide.QtGui import QSlider, QFormLayout
from PySide.QtCore import Qt

# Local modules.
from pyhmsa.gui.spec.datum.datum import _DatumWidget
from pyhmsa.gui.spec.datum.analysis import \
    (Analysis0DTableWidget, Analysis1DGraphWidget, Analysis1DTableWidget,
     Analysis2DGraphWidget, Analysis2DTableWidget)

from pyhmsa.spec.datum.analysislist import \
    AnalysisList0D, AnalysisList1D, AnalysisList2D

# Globals and constants variables.

class _AnalysisListWidget(_DatumWidget):

    def __init__(self, clasz, controller, datum=None, parent=None):
        self._datum = None
        _DatumWidget.__init__(self, clasz, controller, datum, parent)

    def _init_ui(self):
        # Widgets
        self._slider = QSlider(Qt.Orientation.Horizontal)
        self._slider.setTickPosition(QSlider.TicksBelow)

        self._wdg_analysis = self._create_analysis_widget()

        # Layouts
        layout = _DatumWidget._init_ui(self)

        sublayout = QFormLayout()
        sublayout.addRow('Analyses', self._slider)
        layout.addLayout(sublayout)

        layout.addWidget(self._wdg_analysis)

        # Signals
        self._slider.valueChanged.connect(self._onSlide)

        return layout

    def _create_analysis_widget(self):
        raise NotImplementedError

    def _onSlide(self, value):
        if self._datum is None:
            return
        analysis = self._datum.toanalysis(value)
        self._wdg_analysis.setDatum(analysis)

    def setDatum(self, datum):
        _DatumWidget.setDatum(self, datum)
        self._datum = datum
        self._slider.setMaximum(datum.analysis_count - 1)
        self._slider.setValue(0)
        self._onSlide(0)

class AnalysisList0DTableWidget(_AnalysisListWidget):

    def __init__(self, controller, datum=None, parent=None):
        _AnalysisListWidget.__init__(self, AnalysisList0D, controller,
                                     datum, parent)

    def _create_analysis_widget(self):
        return Analysis0DTableWidget(self.controller)

class AnalysisList1DTableWidget(_AnalysisListWidget):

    def __init__(self, controller, datum=None, parent=None):
        _AnalysisListWidget.__init__(self, AnalysisList1D, controller,
                                     datum, parent)

    def _create_analysis_widget(self):
        return Analysis1DTableWidget(self.controller)

class AnalysisList1DGraphWidget(_AnalysisListWidget):

    def __init__(self, controller, datum=None, parent=None):
        _AnalysisListWidget.__init__(self, AnalysisList1D, controller,
                                     datum, parent)

    def _create_analysis_widget(self):
        return Analysis1DGraphWidget(self.controller)

class AnalysisList2DTableWidget(_AnalysisListWidget):

    def __init__(self, controller, datum=None, parent=None):
        _AnalysisListWidget.__init__(self, AnalysisList2D, controller,
                                     datum, parent)

    def _create_analysis_widget(self):
        return Analysis2DTableWidget(self.controller)

class AnalysisList2DGraphWidget(_AnalysisListWidget):

    def __init__(self, controller, datum=None, parent=None):
        _AnalysisListWidget.__init__(self, AnalysisList2D, controller,
                                     datum, parent)

    def _create_analysis_widget(self):
        return Analysis2DGraphWidget(self.controller)



"""
Specimen conditions widgets
"""

# Standard library modules.

# Third party modules.

# Local modules.
from pyhmsa.gui.util.parameter import \
    TextAttributeLineEdit, NumericalAttributeLineEdit
from pyhmsa.gui.spec.condition.condition import _ConditionWidget
from pyhmsa.gui.spec.condition.composition import CompositionElementalWidget

from pyhmsa.spec.condition.specimen import Specimen

# Globals and constants variables.

class SpecimenWidget(_ConditionWidget):

    def __init__(self, parent=None):
        _ConditionWidget.__init__(self, Specimen, parent)

    def _init_ui(self):
        # Widgets
        self._txt_name = TextAttributeLineEdit(self.CLASS.name)
        self._txt_description = TextAttributeLineEdit(self.CLASS.description)
        self._txt_origin = TextAttributeLineEdit(self.CLASS.origin)
        self._txt_formula = TextAttributeLineEdit(self.CLASS.formula)
        self._wdg_composition = CompositionElementalWidget()
        self._txt_temperature = NumericalAttributeLineEdit(self.CLASS.temperature)

        # Layouts
        layout = _ConditionWidget._init_ui(self)
        layout.addRow('<i>Name</i>', self._txt_name)
        layout.addRow('Description', self._txt_description)
        layout.addRow('Origin', self._txt_origin)
        layout.addRow('Formula', self._txt_formula)
        layout.addRow('Composition', self._wdg_composition)
        layout.addRow('Temperature', self._txt_temperature)

        # Signals
        self._txt_name.textEdited.connect(self.edited)
        self._txt_description.textEdited.connect(self.edited)
        self._txt_origin.textEdited.connect(self.edited)
        self._txt_formula.textEdited.connect(self.edited)
        self._wdg_composition.edited.connect(self.edited)
        self._txt_temperature.textEdited.connect(self.edited)

        return layout

    def _create_parameter(self):
        return self.CLASS('dummy')

    def parameter(self, parameter=None):
        parameter = _ConditionWidget.parameter(self, parameter)
        parameter.name = self._txt_name.text()
        parameter.description = self._txt_description.text()
        parameter.origin = self._txt_origin.text()
        parameter.formula = self._txt_formula.text()
        parameter.composition = self._wdg_composition.condition()
        parameter.temperature = self._txt_temperature.text()
        return parameter

    def setParameter(self, condition):
        _ConditionWidget.setParameter(self, condition)
        self._txt_name.setText(condition.name)
        self._txt_description.setText(condition.description)
        self._txt_origin.setText(condition.origin)
        self._txt_formula.setText(condition.formula)
        self._wdg_composition.setCondition(condition.composition)
        self._txt_temperature.setText(condition.temperature)

    def setReadOnly(self, state):
        _ConditionWidget.setReadOnly(self, state)
        self._txt_name.setReadOnly(state)
        self._txt_description.setReadOnly(state)
        self._txt_origin.setReadOnly(state)
        self._txt_formula.setReadOnly(state)
        self._wdg_composition.setReadOnly(state)
        self._txt_temperature.setReadOnly(state)

    def isReadOnly(self):
        return _ConditionWidget.isReadOnly(self) and \
            self._txt_name.isReadOnly() and \
            self._txt_description.isReadOnly() and \
            self._txt_origin.isReadOnly() and \
            self._txt_formula.isReadOnly() and \
            self._wdg_composition.isReadOnly() and \
            self._txt_temperature.isReadOnly()

    def hasAcceptableInput(self):
        return _ConditionWidget.hasAcceptableInput(self) and \
            self._txt_name.hasAcceptableInput() and \
            self._txt_description.hasAcceptableInput() and \
            self._txt_origin.hasAcceptableInput() and \
            self._txt_formula.hasAcceptableInput() and \
            self._wdg_composition.hasAcceptableInput() and \
            self._txt_temperature.hasAcceptableInput()



"""
Access icon themes.
"""

# Standard library modules.
import os
import glob
import warnings

# Third party modules.
from PySide.QtCore import QResource
from PySide.QtGui import QIcon

import pkg_resources

# Local modules.

# Globals and constants variables.

def init_resources():
    dirpath = pkg_resources.resource_filename(__name__, 'icons') #@UndefinedVariable
    for filepath in glob.glob(os.path.join(dirpath, '*.rcc')):
        if not QResource.registerResource(filepath):
            warnings.warn('Could not register rcc: %s filepath')
            continue
    QIcon.setThemeName('pyhmsa')

init_resources()

def getIcon(name):
    if not QIcon.hasThemeIcon(name):
        warnings.warn('Unknown icon theme: %s' % name)
        return QIcon()
    return QIcon.fromTheme(name)
